@@ -132,26 +132,29 @@ class ClientCreator(object):
             return
         S3RegionRedirector(endpoint_bridge, client).register()
         self._set_s3_addressing_style(
-            endpoint_url, client.meta.config.s3, client.meta.events)
+            endpoint_url, client.meta.config.s3, client.meta.events,
+            client.meta.partition
+        )
         # Enable accelerate if the configuration is set to to true or the
         # endpoint being used matches one of the accelerate endpoints.
         if self._is_s3_accelerate(endpoint_url, client.meta.config.s3):
             # Also make sure that the hostname gets switched to
             # s3-accelerate.amazonaws.com
             client.meta.events.register_first(
-                'request-created.s3', switch_host_s3_accelerate)
+                'before-sign.s3', switch_host_s3_accelerate)
 
         self._set_s3_presign_signature_version(
             client.meta, client_config, scoped_config)
 
-    def _set_s3_addressing_style(self, endpoint_url, s3_config, event_emitter):
+    def _set_s3_addressing_style(self, endpoint_url, s3_config, event_emitter,
+                                 partition):
         if s3_config is None:
             s3_config = {}
 
         addressing_style = self._get_s3_addressing_style(
             endpoint_url, s3_config)
         handler = self._get_s3_addressing_handler(
-            endpoint_url, s3_config, addressing_style)
+            endpoint_url, s3_config, addressing_style, partition)
         if handler is not None:
             event_emitter.register('before-sign.s3', handler)
 
@@ -168,7 +171,7 @@ class ClientCreator(object):
             return configured_addressing_style
 
     def _get_s3_addressing_handler(self, endpoint_url, s3_config,
-                                   addressing_style):
+                                   addressing_style, partition):
         # If virtual host style was configured, use it regardless of whether
         # or not the bucket looks dns compatible.
         if addressing_style == 'virtual':
@@ -186,12 +189,6 @@ class ClientCreator(object):
         logger.debug("Defaulting to S3 virtual host style addressing with "
                      "path style addressing fallback.")
 
-        # For dual stack mode, we need to clear the default endpoint url in
-        # order to use the existing netloc if the bucket is dns compatible.
-        if s3_config.get('use_dualstack_endpoint', False):
-            return functools.partial(
-                fix_s3_host, default_endpoint_url=None)
-
         # By default, try to use virtual style with path fallback.
         return fix_s3_host
 
@@ -618,6 +615,16 @@ class BaseClient(object):
 
     def _convert_to_request_dict(self, api_params, operation_model,
                                  context=None):
+        api_params = self._emit_api_params(
+            api_params, operation_model, context)
+        request_dict = self._serializer.serialize_to_request(
+            api_params, operation_model)
+        prepare_request_dict(request_dict, endpoint_url=self._endpoint.host,
+                             user_agent=self._client_config.user_agent,
+                             context=context)
+        return request_dict
+
+    def _emit_api_params(self, api_params, operation_model, context):
         # Given the API params provided by the user and the operation_model
         # we can serialize the request to a request_dict.
         operation_name = operation_model.name
@@ -639,13 +646,7 @@ class BaseClient(object):
                 endpoint_prefix=self._service_model.endpoint_prefix,
                 operation_name=operation_name),
             params=api_params, model=operation_model, context=context)
-
-        request_dict = self._serializer.serialize_to_request(
-            api_params, operation_model)
-        prepare_request_dict(request_dict, endpoint_url=self._endpoint.host,
-                             user_agent=self._client_config.user_agent,
-                             context=context)
-        return request_dict
+        return api_params
 
     def get_paginator(self, operation_name):
         """Create a paginator for an operation.
